# FIACRE.tcl--- Oxford FIACRE module control frame
#  

set frame FIACRE
set frametitle "FIACRE module Setup & Monitor"
set icontitle "FIACRE"

set mname "FIACRE"

set maxmodule [inform camaccontrol inquire [set mname]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [first $modulelist]

set maxadc 0
set nchannels 0

set rrname(INCRIR)      incrIR
set rrname(SETI)        setI

set rrname(DCENABLE)    DCEnable
set rrname(DCDISABLE)   DCDisable

set rrname(DIAGMENABLE)  DiagMEnable
set rrname(DIAGMDISABLE) DiagMDisable

set rrname(RESET)       reset

set rrname(STATUS)	status
set rrname(STATUSIR)	status.IR
set rrname(STATUSCA)	status.CAddr
set rrname(STATUSCN)	status.N
set rrname(STATUSSW)	status.CMSwitch
set rrname(STATUSDC)	status.DC
set rrname(STATUSDM)	status.DM


for {set sid [array startsearch rrname]} {[array anymore rrname $sid]} { } {
	set element [array nextelement rrname $sid]
	set wname($rrname($element)) $element
        set wrname($element) $rrname($element)
}
for {set sid [array startsearch wrname]} {[array anymore wrname $sid]} { } {
	set element [array nextelement wrname $sid]
	set wname($wrname($element)) $element
}


loadsource common.camac
set reset_cmd "reset"


proc repaint {} {
  global module
  :modify MODULE-NUM :value $module
}

proc redisplay-module {} {
   global debug
   if {$debug} then {puts stderr "redisplay-module with arguments"}
   camac-address
   foreach register [list status.IR status.CAddr status.N status.CMSwitch status.DC status.DM] {
      update-widget $register [read-reg $register]
   }
}

proc click arg {
   global debug
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      MODULE-SEL-MENU -
      MODULE-SEL      -
      MODULE-NUM {click-select-module $v}

      RESET  -
      SETI   -
      INCRIR    -
      DCENABLE    -
      DCDISABLE    -
      DIAGMENABLE   -
      DIAGMDISABLE 	{click-action $w}

      REDISPLAY	{click-redisplay}
      HELP      {click-help}

      default   {puts stdout $arg}
   }
}


proc make-frame {} {
  global env maxmodule camcrate camstation
  global modulelist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]"

   :panel MODULE :layout T  :padx 5 :pady 5
      :layout H; :row-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5
      :menubutton MODULE-SEL :label "select:" :strings $modulelist
      :hskip 50
      :text CAM-CRATE   :label "CAMAC Crate:"   :w 5 :value $camcrate   :deaf T
      :text CAM-STATION :label "CAMAC Station:" :w 5 :value $camstation :deaf T

      :next-row T
      :message M1 :label "Status Register" :bold T
      :row-gap 0; :next-row T
      :text STATUSIR :label "Interrupt Register" :w 4 :deaf T
      :text STATUSCA :label "Channel Address"    :w 4 :deaf T
      :text STATUSCN :label "Camac Station"      :w 4 :deaf T
      :checkbox STATUSSW :label "Camac/Manual Switch" :deaf T
      :checkbox STATUSDC :label "Dummy Cycle"         :deaf T
      :checkbox STATUSDM :label "Diagnostic Mode"     :deaf T

      :row-gap 10
      :next-row T; :col-gap 10
      :button RESET      :label "Reset"
      :button SETI       :label "Set Internal Interrupt"
      :button INCRIR     :label "Increment Interrupt Register"

      :button DCENABLE   :label "Enable Dummy Cycle"
      :button DCDISABLE  :label "Disable Dummy Cycle"
      :next-row T
      :button DIAGMENABLE   :label "Enable Diagnostic Mode"
      :button DIAGMDISABLE  :label "Disable Diagnostic Mode"

      :next-row T
      :button REDISPLAY; :button HELP

   :show T

}

#proc EG args {insert-debug "$args"; return 7}
