# amps.tcl---MEGHA amplifier control frame


set frame AMPS
set frametitle "MEGHA Amplifier Setup & Monitor"
set icontitle "Amps"

set started 0

if {[info exists env(VmeServer)]} then {set resource $env(VmeServer)} \
    else {set resource vme}


set savedirectory MeghaAmpSetup

set cratenum 1
set ampnum 1
set allcrates 0
set allamps 0
set ampsel 1
set cratesel 1
set ampselection {1}
set crateselection {1}

set timec 0
set cgain 0
set thresh 0
set fgain 0
set gtime 0

set timecmenu [list "2us (short)" "10us (long)"]
set cgainmenu [list 5 10 25 50 100 250 500 1000]

set hexchars [list 0 1 2 3 4 5 6 7 8 9 A B C D E F]

#    for debugging using crate 1
#set hexchars [list 0 0 0 0 0 0 0 0 0 0 A B C D E F]

set selectcratemenu {1 2 3 4 5 6 7 8 9 10}
set selectampmenu   {1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32}
set cratelist $selectcratemenu
set amplist $selectampmenu

set cratepresent(1) 0
set cratepresent(2) 0
set cratepresent(3) 0
set cratepresent(4) 0
set cratepresent(5) 0
set cratepresent(6) 0
set cratepresent(7) 0
set cratepresent(8) 0
set cratepresent(9) 0
set cratepresent(10) 0

set AmpStatus ""
set Status ""
set ampcmd {}

proc start args {
   global started
   if {$started == 0} then {make-frame; EGbindings} else {open-frame}
   set started 1
   set-footer "Probing for amplifier crates"
   probe-for-crates
   clear-busy
   set-footer "Ready"
   show-frame
}

proc define-module {} {
  global resource
  set-footer "deleting all Megha Amp registers"
  EG "register :pattern '/$resource/MeghaAmp.*' !delete"

#    write command (without response)
  define-register  TxCMD      "-p wo"

#    write command (with response)
  define-register  CMD        "-p rw"
}

proc define-register {name attributes} {
   global resource
   set register /$resource/MeghaAmp.$name
   set-footer "defining register $register"
   catch {EG "register :name '$register' :class 'xMeghaAmp' !new"}
   if {$attributes != ""} then {EG "register :name '$register' :object '$attributes'"}
   clear-footer
}

proc write-register {name value} {
   global resource
   set register /$resource/MeghaAmp.$name
   EG "register :name '$register' :value '$value'"
}

proc read-register {name} {
   global resource
   set register /$resource/MeghaAmp.$name
   set value [EG "register :name '$register' ?value"]
   return $value
}

proc threshold v {
  global thresh
#    v is in units of mV - convert to hardware format using (T/5)+3
  set thresh [expr [expr $v / 5] + 3]
#    number in range 4 to 99
  if {$thresh < 4} then {set thresh 4}
  if {$thresh > 99} then {set thresh 99}
  if {$thresh < 10} then {set thresh 0$thresh}
  do-cmd "T$thresh"
}

proc fine-gain v {
  global fgain
#    v is converted to "physical units" using 500/F
  set fgain [expr round([expr 500 / $v])]
#    number in range 200 to 600
  if {$fgain < 200} then {set fgain 200}
  if {$fgain > 600} then {set fgain 600}
  do-cmd "F$fgain"
}

proc gatetime v {
  global gtime
#    v is in units of us - convert to hardware format using (120-4G)
  set gtime [expr round([expr 120 - [expr $v * 4]])]
#    number in range 0 to 99
  if {$gtime < 0} then {set gtime 0}
  if {$gtime > 99} then {set gtime 99}
  if {$gtime < 10} then {set gtime 0$gtime}
  do-cmd "G$gtime"
}

proc relay {} {
   global timec cgain
   do-cmd "R$timec$cgain"
}

proc do-cmd {s} {
   global ampcmd
   global allcrates allamps cratepresent
   global cratenum selectcratemenu crateselection
   global ampnum selectampmenu ampselection
   global hexchars
   if {$allcrates == 0} then {set cratelist [list $cratenum]}
   if {$allcrates == 1} then {set cratelist $selectcratemenu}
   if {$allcrates == -1} then {set cratelist $crateselection}
   if {$allamps == 0} then {set amplist [list $ampnum]}
   if {$allamps == 1} then {set amplist $selectampmenu}
   if {$allamps == -1} then {set amplist $ampselection}

   foreach crate $cratelist {
      if {$cratepresent($crate) != 0} then {
         set cmd ""
         foreach amp $amplist {
            if {$amp > 9} then {set aa $amp} else {set aa 0$amp}
            set cc [lindex $hexchars [expr $crate - 1]]
            append cmd "#[set cc][set aa]$s\r"
         }
         write-register TxCMD "$cmd"
      }
   }
   do-update
}

proc crate v {
   global cratenum cratepresent
   if {$cratepresent($v) == 0} then {
      midas-warning "Crate $v may not be connected - Use Probe command if you think it is connected to check for existance"
      repaint
      return
   }
   set cratenum $v
   do-Mcmd
   do-update
}

proc amplifier v {
   global ampnum
   set ampnum $v
   do-Mcmd
   do-update
}

proc do-Mcmd {} {
   global cratenum
   global ampnum
   global hexchars

   set crate $cratenum
   set amp $ampnum
   if {$amp > 9} then {set aa $amp} else {set aa 0$amp}
   set cc [lindex $hexchars [expr $crate - 1]]
   set cmd "#[set cc][set aa]M"
   write-register TxCMD "$cmd\r"
}

proc probe-for-crates {} {
#    probe for crates to check presence
   global EGallow 
   global selectcratemenu cratepresent cratenum ampnum
   global hexchars

   define-module

   set cratenum 0
   set ampnum 1
   set-busy
   set pattern 0
   :modify CRATESINUSE :value $pattern
   foreach crate $selectcratemenu {
      set cc [lindex $hexchars [expr $crate - 1]]
      set cmd "#[set cc]01V"
      set-footer "Probing for crate $crate"
      set EGallow 0x2000a
      write-register CMD "$cmd\r"
      set reply [read-register CMD]
      set EGallow -1
      if {[string length $reply] > 0} then {
#    response (of some sort) from crate
          set cratepresent($crate) 1
          if {$cratenum == 0} then {set cratenum $crate}
          incr pattern [expr 1 << [expr $crate - 1]]
      } else {
          set cratepresent($crate) 0
      }
   :modify CRATESINUSE :value $pattern
   }
   clear-busy
   clear-footer
   if {$cratenum == 0} then {
      midas-warning "No amplifier crates found!"
   } else {
      do-update
   }
}


proc do-update {} {
   global AmpStatus
   global cratenum ampnum
   global timec cgain thresh gtime fgain

   set count 0
   while {[do_Vcmd $cratenum $ampnum] == 1} {
      incr count
      if {$count > 5} then {
         midas-warning "Repeated bad response from Amplifier\nYou could try the Reset Interface button"
         return
      }
   }
   if {"$AmpStatus" == ""} then {return}

#    update data values

   set timec [scan-int [string range $AmpStatus 4 4]]
   :modify TIMEC :value $timec
   set cgain [scan-int [string range $AmpStatus 5 5]]
   :modify CGAIN :value $cgain
   set thresh [scan-int [string range $AmpStatus 7 8]]
   if {$thresh < 4} then {set thresh 4}
   if {$thresh > 99} then {set thresh 99}
#    convert from hardware units to mV using (T-3)*5
   :modify THRESHOLD :value [expr [expr $thresh - 3] * 5]
   set gtime [scan-int [string range $AmpStatus 10 11]]
   if {$gtime < 0} then {set gtime 0}
   if {$gtime > 99} then {set gtime 99}
#    convert from hardware units to us using (120-G)/4
   :modify GATETIME :value [expr [expr 120 - $gtime] / 4.0]
   set fgain [scan-int [string range $AmpStatus 13 15]]
   if {$fgain < 200} then {set fgain 200}
   if {$fgain > 600} then {set fgain 600}
#    convert from hardware units using 500/F
   :modify FGAIN :value [expr [expr round([expr 500.0 / $fgain * 10000])] / 10000.0]
}

proc show-status {} {
   global env AmpStatus
   global selectcratemenu cratenum selectampmenu timecmenu cgainmenu
   global cratepresent allcrates crateselection

   if {$allcrates == 0} then {set cratelist [list $cratenum]}
   if {$allcrates == 1} then {set cratelist $selectcratemenu}
   if {$allcrates == -1} then {set cratelist $crateselection}

   set contents ""
   set-busy
   foreach crate $cratelist {
      if {$cratepresent($crate) != 0} then {
         set-footer "Getting amplifier settings from crate $crate..."
         append contents "Crate $crate\nAmplifier #  --  time constant;  coarse gain;  threshold;  gate time;  fine gain\n"
         foreach amp $selectampmenu {
#    obtain values for this amplifier
            if {$amp > 9} then {set aa $amp} else {set aa 0$amp}
            set count 0
            while {[do_Vcmd $crate $amp] == 1} {
               incr count
               if {$count > 5} then {
                  midas-warning "Repeated bad response from Amplifier\nYou could try the Reset Interface button"
                  set AmpStatus ""
                  break
               }
            }
            if {"$AmpStatus" == ""} then {break}

            set timec [scan-int [string range $AmpStatus 4 4]]
            set cgain [scan-int [string range $AmpStatus 5 5]]
            set thresh [scan-int [string range $AmpStatus 7 8]]
            if {$thresh < 4} then {set thresh 4}
            if {$thresh > 99} then {set thresh 99}

            set gtime [scan-int [string range $AmpStatus 10 11]]
            if {$gtime < 0} then {set gtime 0}
            if {$gtime > 99} then {set gtime 99}
            set fgain [scan-int [string range $AmpStatus 13 15]]
            if {$fgain < 200} then {set fgain 200}
            if {$fgain > 600} then {set fgain 600}
            set utimec [lindex $timecmenu $timec]
            set ucgain [lindex $cgainmenu $cgain]
            set uthresh [expr [expr $thresh - 3] * 5]
            set ugtime [expr [expr 120 - $gtime] / 4.0]
            set ufgain [expr [expr round([expr 500.0 / $fgain * 1000])] / 1000.0]
            append contents "Amp $aa  --   $utimec     $ucgain     $uthresh mV     $ugtime us     $ufgain \n"
         }
      }
   }
   midas-information $contents
   clear-busy
   clear-footer
}

proc do_Vcmd {crate amp} {
   global AmpStatus
   global hexchars

   if {$amp > 9} then {set aa $amp} else {set aa 0$amp}
   set cc [lindex $hexchars [expr $crate - 1]]
   set cmd "#[set cc][set aa]V"

   write-register CMD "$cmd\r"
   set reply [read-register CMD]

#    now analyse reply
#    expect nnnRnnTnnGnnFnnn - total length 16 characters

   if {[string length $reply] == 0} then {
      midas-warning "No response from Crate $crate - check connection and power"
      set AmpStatus ""
      return 0
   }

   if {[string length $reply] != 16} then {return 1}
   if {"[string range $reply 0 2]" != "[set cc][set aa]"} then {return 1}
   if {"[string range $reply 3 3]" != "R"} then {return 1}
   if {"[string range $reply 6 6]" != "T"} then {return 1}
   if {"[string range $reply 9 9]" != "G"} then {return 1}
   if {"[string range $reply 12 12]" != "F"} then {return 1}

#    response structure OK
   set AmpStatus $reply
   return 0
}

proc badreply s {midas-warning "Amplifier response received $s is not as expected"}

proc repaint {} {
   global allcrates allamps
   global cratenum ampnum
  :modify CRATE-NUM :value $cratenum
  if {$allcrates == -1} then {
     :modify ALL-CRATES :value 2
  } else {
     :modify ALL-CRATES :value $allcrates
  }
  :modify AMP-NUM   :value $ampnum
  if {$allamps == -1} then {
     :modify ALL-AMPS  :value 2
  } else {
     :modify ALL-AMPs :value $allamps
  }
}

proc click-save {w} {
   global env EGreport AmpStatus Status savedirectory
   global selectcratemenu selectampmenu cratenum
   global cratepresent

   catch {file delete [file join $env(EXPERIMENTDIR) $savedirectory]}
   if {![file exists [file join $env(EXPERIMENTDIR) $savedirectory]]} then {
      set z [catch {:file mkdir [file join $env(EXPERIMENTDIR) $savedirectory]} m]
      if {$z != 0} then {midas-warning "Cannot create directory [file join $env(EXPERIMENTDIR) $savedirectory]\n $m"}
   }

   if {$w == "ALL"} then {
      set cratelist $selectcratemenu
   } else {
      set cratelist $cratenum
   }
   set amplist $selectampmenu

   set-busy
   set EGreport_SAVE $EGreport
   set EGreport -1

   set contents ""
   foreach crate $cratelist {
      set Status {}
      if {$cratepresent($crate) != 0} then {
         set-footer "Saving amplifier settings from crate $crate..."
         append contents "Amplifier settings saved from crate $crate amplifiers"
         foreach amp $amplist {
#    obtain values for this amplifier
            set count 0
            while {[do_Vcmd $crate $amp] == 1} {
               incr count
               if {$count > 5} then {
                  midas-warning "Repeated bad response from Amplifier\nYou could try the Reset Interface button"
                  set AmpStatus  ""
                  break
               }
            }
            if {"$AmpStatus" != ""} then {
               lappend Status [string range $AmpStatus 0 15]
               append contents " $amp"
            } else {
               break
            }
         }
#    save amplifier status to file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory c$crate] w]
         puts $f $Status
         close $f
         append contents " to file [file join $env(EXPERIMENTDIR) $savedirectory c$crate]\n"
      }
   }

   set EGreport $EGreport_SAVE
   clear-busy

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been saved!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings saved to $env(EXPERIMENTDIR)/$savedirectory"

   do-update
}

proc click-restore {w} {
     global env EGreport
     global AmpStatus Status savedirectory
     global cratenum selectcratemenu cratepresent hexchars

     if {$w == "ALL"} then {
        set cratelist $selectcratemenu
     } else {
        set cratelist $cratenum
     }

     set-busy
     set EGreport_SAVE $EGreport
     set EGreport -1
     set contents ""

     foreach crate $cratelist {
       if {$cratepresent($crate) != 0} then {
         set-footer "Restoring Amplifier settings to crate $crate..."
         append contents "Amplifier settings restored to crate $crate amplifiers"

         set f [open [file join $env(EXPERIMENTDIR) $savedirectory c$crate] r]
         set Status [gets $f]
         close $f

         foreach AmpStatus $Status {

            set aa [string range $AmpStatus 1 2]
            if {$aa > 9} then {set amp $aa} else {set amp [string range $aa 1 1]}
            set cc [lindex $hexchars [expr $crate - 1]]

            set timec [string range $AmpStatus 4 4]
            set cgain [string range $AmpStatus 5 5]
            set cmd "#[set cc][set aa]R$timec$cgain\r"

            set thresh [string range $AmpStatus 7 8]
            append cmd "#[set cc][set aa]T$thresh\r"

            set gtime [string range $AmpStatus 10 11]
            append cmd "#[set cc][set aa]G$gtime\r"

            set fgain [string range $AmpStatus 13 15]
            append cmd "#[set cc][set aa]F$fgain\r"

            write-register TxCMD "$cmd"
            append contents " $amp"
         }
         append contents " from file [file join $env(EXPERIMENTDIR) $savedirectory c$crate]\n"
       }
     }

     set EGreport $EGreport_SAVE
     clear-busy

     if {$contents == ""} then {
       midas-warning "Warning - Nothing has been restored!"
     } else {
       midas-report "$contents"
     }

     set-footer "Amplifier settings restored from $env(EXPERIMENTDIR)/$savedirectory"

     do-update
}

proc click-setall {} {
    global cratenum ampnum AmpStatus hexchars
    global timec cgain thresh fgain gtime

    do_Vcmd $cratenum $ampnum

    set timec [string range $AmpStatus 4 4]
    set cgain [string range $AmpStatus 5 5]
    set thresh [string range $AmpStatus 7 8]
    set gtime [string range $AmpStatus 10 11]
    set fgain [string range $AmpStatus 13 15]

    set aa [string range $AmpStatus 1 2]
    set cc [lindex $hexchars [expr $cratenum - 1]]

    set cmd    "#[set cc][set aa]R$timec$cgain\r"
    append cmd "#[set cc][set aa]T$thresh\r"
    append cmd "#[set cc][set aa]G$gtime\r"
    append cmd "#[set cc][set aa]F$fgain\r"

    if { $timec == 0 } then {set timec 1} else {set timec 0}
    if { $cgain == 0 } then {set cgain 1} else {set cgain [expr $cgain - 1]}
    relay
    if { $thresh > 50 } then {threshold 100} else {threshold 400}
    if { $gtime > 50 } then {gatetime 10} else {gatetime 25}
    if { $fgain > 300 } then {fine-gain 1.0} else {fine-gain 2.0}

    write-register TxCMD "$cmd"

    click-redisplay

    set-footer "Brute force set complete."
}

proc click-redisplay {} {repaint; do-update}

proc click arg {
   global debug
   global cratenum ampnum allcrates allamps
   global timec cgain
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      CRATE-SEL    -
      CRATE-NUM    {crate $v}
      CRATESEL     {click-crateselection $v}

      ALL-CRATES   {click-whichcrates $v}

      AMP-SEL      -
      AMP-NUM      {amplifier $v}
      AMPSEL       {click-ampselection $v}

      ALL-AMPS     {click-whichamps $v}

      TIMEC        {set timec $v; relay}
      CGAIN        {set cgain $v; relay}

      THRESHOLD    {threshold $v}
      GATETIME     {gatetime $v}
      FGAIN        {fine-gain $v}

      PROBE        {probe-for-crates}
      RESET        {reset-interface}
      STATUS       {show-status}

      SETALL     {click-setall}

      SAVE         {click-save CURRENT}
      SAVEALL      {click-save ALL}
      RESTORE      {click-restore CURRENT}
      RESTOREALL   {click-restore ALL}

      REDISPLAY    {click-redisplay}
      default      {puts stdout $arg}
   }
   repaint
}


proc make-frame {} {
  global env
  global selectcratemenu selectampmenu timecmenu cgainmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/charissa" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number CRATE-NUM  :label "Crate number:" :min 1 :max 16 :w 5  :valuex 16c
      :menubutton CRATE-SEL :label "select:" :strings $selectcratemenu
      :hskip 10
      :choice ALL-CRATES :label "Act on crates?" :valuex 18c :strings [list Current All Select] :value 0
      :next-row T
      :number AMP-NUM  :label "Amplifier number:" :min 1 :max 32 :w 5 :valuex 16c
      :menubutton AMP-SEL :label "select:" :strings $selectampmenu
      :hskip 10
      :choice ALL-AMPS :label "Act on amplifiers?" :valuex 18c :strings [list Current All Select] :value 0
      :next-row T
      :message M1 :label "Crates in use" :bold T
      :hskip 30
      :choice CRATESINUSE :label "" :exclusive F :strings $selectcratemenu :value 0 :deaf T

    :panel P1 :layout L :padx 10
      :layout H; :col-gap 15; :row-gap 15

      :message M2 :label "Relays" :bold T
      :choice TIMEC  :label "Time Constant:" :exclusive T  :strings $timecmenu :value 0
      :choice-stack CGAIN  :label "Coarse Gain:" :exclusive T :strings $cgainmenu  :value 0
      :next-row T
      :slider THRESHOLD :label "Threshold (mV)" :min 5 :max 500 :resolution 5 :valuex 30c
      :next-row T
      :slider GATETIME  :label "Gate Time (us)" :min 5 :max 30 :resolution 0.25  :valuex 30c
      :next-row T
      :slider FGAIN     :label "Fine Gain" :min 0.8 :max 2.5 :resolution 0.002 :valuex 30c

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :col-gap 10; :row-gap 5
      :button PROBE    :label "Probe for crates"
      :button STATUS   :label "Show Settings"
      :next-row T
      :col-gap 0
      :button SAVEALL     :label "Save All Crates"
      :button SAVE        :label "Save Current Crate"
      :button RESTOREALL  :label "Restore All Crates"
      :button RESTORE     :label "Restore Current Crate"
      :next-row T
      :button REDISPLAY
      :hskip 50
      :button SETALL :label "Reload current Amp settings"
      :hskip 50
      :button HELP  :command click-help

      :show T

}

proc make-crateSELframe {} {
  global cratesel cratelist

   :frame CRATESEL-FRAME :label "Amplifier Crate Selection"  :show-footer F :resize F :transient T
   :panel CRATESEL-PANEL :layout T
   :choice CRATESEL :exclusive F :strings $cratelist :value $cratesel
   :show T
}

proc make-ampSELframe {} {
  global ampsel amplist

   :frame AMPSEL-FRAME :label "Amplifier Amp Selection"  :show-footer F :resize F :transient T
   :panel AMPSEL-PANEL :layout T
   :choice AMPSEL :exclusive F :strings $amplist :value $ampsel
   :show T
}

proc click-whichamps v {
  global allamps
  switch $v {
    0  -
    1  {set allamps $v}
    2  {
       set allamps -1
       make-ampSELframe
    }
  }
}

proc click-whichcrates v {
  global allcrates
  switch $v {
    0  -
    1  {set allcrates $v}
    2  {
       set allcrates -1
       make-crateSELframe
    }
  }
}

proc click-ampselection v {
  global ampsel amplist ampselection
  set ampsel $v
  set ampselection {}
  foreach i [list-take $amplist [bits-in $ampsel]] {
    lappend  ampselection $i
  }
}

proc click-crateselection v {
  global cratesel cratelist crateselection
  set cratesel $v
  set crateselection {}
  foreach i [list-take $cratelist [bits-in $cratesel]] {
    lappend crateselection  $i
  }
}
