#    f2vbdiag.tcl  -  F2VB module diagnostic frame

set DataWidth 32

set dumpopt 1
set dumpwords 256

set memdisplaysize 32

proc add-format-option {} {
     global dumpopt
     :choice-stack FORMATOPT :label "format option" :strings [list raw "format as Megha"] :value $dumpopt
}

proc format_the_dump {} {
  global dumpdata
  global dumpopt dumpstart dumpwords

#    format f2vb data RAM data held in global variable dumpdata

   set-footer "formatting dump"

   set frame "dump"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted Data Dump"  :transient T
   :layout T

   switch $dumpopt {
   0  {
        :galley G :rows 10 :cols 100 :deaf T :font fixed :strings ""
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 50 :cols 75 :deaf T :font fixed :strings ""

        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]

            if {$word == 0xffff0000} then {
               :modify G :strings "+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+\n"
               break
           }

            if {[format 0x%x [expr $word & 0xffff0000]] == 0xffff0000} then {
               set wc1 [expr $word & 0x0000ffff]
               set wc2 [expr ($wc1 - 2) / 2]
               :modify G :strings "-  Event Header  WC = $wc1;  data items = $wc2 --------------------------\n"
               continue
           }

            if {[format 0x%x [expr $word & 0xe000e000]] != 0x20008000} then {
               :modify G :strings "Bad Data Word $word \n" 
               continue
           }

           set item [expr ($word >> 16) & 1]
           set channel [expr (($word >> 17) & 31) + 1]
           set module [expr (($word >> 22) & 3) + 1]
           set crate [expr ($word >> 24) & 31]
           set adc [expr $word & 0x00001fff]

           :modify G :strings "Megha Data Item $word  [format "crate=%d module=%d channel=%2d item=%d data=0x%x" $crate $module $channel $item $adc]\n"
        }
     }
   }
   :modify G :see 0
   :layout B; :set fill none; :set anchor center
   :button B  :label "Dismiss" :command "destroy $win"
   :show T
   clear-footer
}


proc format_a_block {} {
  global dumpdata dumpwords
 
#    run the sort spy test program to capture a block

   set-footer "Obtaining a data block from the sort-spy"
   set dumpdata [exec sort-spy-dump 1]
   set dumpwords [llength $dumpdata]

   set-footer "formatting data block"

   set frame "format"
   :frame [self frame-name].$frame :label  "Formatted Data Block"   :toplevel T

      :galley G :rows 50 :cols 75 :deaf T :font fixed :strings ""

        for {set i 5} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]

            if {$word == 0xffff0000} then {
               :modify G :strings "+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+_+\n"
               break
           }

            if {[format 0x%x [expr $word & 0xffff0000]] == 0xffff0000} then {
               set wc1 [expr $word & 0x0000ffff]
               set wc2 [expr ($wc1 - 2) / 2]
               :modify G :strings "-  Event Header  WC = $wc1;  data items = $wc2 --------------------------\n"
               continue
           }

            if {[format 0x%x [expr $word & 0xe000e000]] != 0x20008000} then {
               :modify G :strings "Bad Data Word $word \n" 
               continue
           }

           set item [expr ($word >> 16) & 1]
           set channel [expr (($word >> 17) & 31) + 1]
           set module [expr (($word >> 22) & 3) + 1]
           set crate [expr ($word >> 24) & 31]
           set adc [expr $word & 0x00001fff]

           :modify G :strings "Megha Data Item $word  [format "crate=%d module=%d channel=%2d item=%d data=0x%x" $crate $module $channel $item $adc]\n"
        }

   :modify G :see 0
   :layout B; :set fill none; :set anchor center
   :button B  :label "Dismiss" :command "destroy $win"
   :show T
   clear-footer
}

proc do_readRAM {} {
  global resource mname DataWidth
  global x2VBmemOffset meminc Doption
  global dumpdata dumplist dumpwords

  if {$Doption == 0} then {
     set register /$resource/$mname.Dbram
     set reg Dbram
     set len $dumpwords
  } else {
     set register /$resource/$mname.Dlram
     set reg Dlram
     if {$Doption == 2} {set len 8} else {set len 96}
  }

  :modify MEMDISPLAY :strings "\n  Memory Offset $x2VBmemOffset\n"

        if {$Doption == 0} then {
            EG "register :name '$register' :object '-o [expr $x2VBmemOffset + 0] -l 64'"
            set data [string trimleft [read-register $reg]]
            EG "register :name '$register' :object '-o [expr $x2VBmemOffset + (64*1*4)] -l 64'"
            append data [string trimleft [read-register $reg]]
            EG "register :name '$register' :object '-o [expr $x2VBmemOffset + (62*2*4)] -l 64'"
            append data [string trimleft [read-register $reg]]
            EG "register :name '$register' :object '-o [expr $x2VBmemOffset + (62*3*4)] -l 64'"
            append data [string trimleft [read-register $reg]]
        } else {
           EG "register :name '$register' :object '-o $x2VBmemOffset -l $len'"
           set data ""
           set i 0
           foreach word [string trimleft [read-register $reg]] {
              append data "$word "
              incr i
              if {$i == 8} then {append data "\n"; set i 0}
           }
        }

  :modify MEMDISPLAY :strings "$data"
  if {$meminc != 0} then {
     set x2VBmemOffset [expr $x2VBmemOffset + $len]
     :modify MEMOFFSET :value $x2VBmemOffset
  }
  if {$Doption == 0} then {set dumpdata $data} else {set dumplist $data}
}
